/*
  File: QueueADT.h
  ----------------
  This file contains the interface of the queue abstract data type.
*/

#ifndef QUEUE_ADT_H
#define QUEUE_ADT_H 1


/*
  Define the abstract data type as a pointer to the concrete data type
*/

typedef struct QueueCDT* QueueADT;



/*
  Function: NewQueue
  Usage: q = NewQueue();
  ----------------------
  Creates a new empty queue using dynamic memory allocation. If the
  dynamic memory allocation is not successful NULL is returned.
*/

QueueADT NewQueue(void);


/*
  Function: FreeQueue
  Usage: FreeQueue(q);
  --------------------
  Frees the memory associated with the queue q. If the queue is not
  empty all its elements are deleted.
  If q is NULL the program terminates with exit code 1.
*/

void FreeQueue(QueueADT);


/*
  Function: Enqueue
  Usage: Enqueue(q, (void*)(&item));
  ----------------------------------
  Inserts the element "item" at the end of the queue q.
  If q is NULL the program terminates with exit code 1.
*/

void Enqueue(QueueADT, void*);


/*
  Function: DeQueue
  Usage: p = DeQueue(q);
  ----------------------
  Removes the element at the front of the queue q. The corresponding
  data is returned.
  If q is NULL the program terminates with exit code 1.
 */

void* DeQueue(QueueADT);

/*
  Function: QueueLength
  Usage: len = QueueLength(q);
  ----------------------------
  Returns the length of the queue q, i.e., the number of elements in q.
  If q is NULL the program terminates with exit code 1.
 */

int QueueLength(QueueADT);


/*
  Function: PrettyPrintQueue
  Usage: PrettyPrintQueue(q);
  ---------------------------
  Print, in a nice way, the length of the queue and the queue elements.
  The queue elements are printed using a printer-function passed via a
  function pointer.
  If q is NULL the program terminates with exit code 1.
 */

void PrettyPrintQueue(QueueADT, void (*print_element)(void*));

#endif /* QUEUE_ADT_H */
